/*
 * SelectListener.java
 *
 * Created on August 27, 2006, 11:31 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm;

import javax.media.opengl.*;
import sddm.modules.*;
/**
 *
 * @author Erik J. Tollerud
 */
public abstract class SelectListener implements GLEventListener {
        private boolean recompile;
        private boolean selectionNeeded;
        private Integer selectDisplayList;
        private float resultf;
        private int resulti;
        private int selectX;
        private int selectY;
        private SDDModule mod;
        protected Coloring indexColorer;
        protected GLAutoDrawable selectDrawer;
        
        private static final float[] BGCOLOR = {1.0f,1.0f,1.0f,1.0f};
        public static final float BGFLOATCOLOR = Coloring.getIndexedFloat(BGCOLOR);
        public static final int BGINTCOLOR = Coloring.getIndexedInt(BGCOLOR);
        
        public SelectListener(SDDModule mod, GLAutoDrawable drawable) {
            /*TODO: figure out why this doesn't work...
            GL gl = drawable.getGL();
            java.nio.IntBuffer compBuffer = java.nio.IntBuffer.allocate(4);
            gl.glGetIntegerv(GL.GL_RED_BITS,compBuffer);
            gl.glGetIntegerv(GL.GL_GREEN_BITS,compBuffer);
            gl.glGetIntegerv(GL.GL_BLUE_BITS,compBuffer);
            gl.glGetIntegerv(GL.GL_ALPHA_BITS,compBuffer);
            System.err.println("Color Component sizes:"+compBuffer.get()+","+compBuffer.get()+","+compBuffer.get()+","+compBuffer.get());
            compBuffer.rewind();
            if (compBuffer.get() != 8 || compBuffer.get() != 8 || compBuffer.get() != 8 || compBuffer.get() != 8 )
                throw new SDDMException("Supplied buffer for selection does not have 8 bits of each RGBA component - can't select");
            */
            
            this.mod = mod;
            selectDrawer = drawable;
            selectDisplayList = null;
            selectionNeeded = false;
            recompile = true;
            resultf = 0;
            resulti = 0;
            selectX=1;
            selectY=1;
            indexColorer = new Coloring();
            indexColorer.setColorSchemeToIndexValue();
            selectDrawer.addGLEventListener(this);
            
            //test for seeing
            //net.java.joglutils.GLJFrame gljf = new net.java.joglutils.GLJFrame(this,drawable.getWidth(),drawable.getHeight());
            //selectDrawer = gljf.getAutoDrawable();
            //gljf.setVisible(true);
        }
        
        public SelectListener(SDDModule mod, int width, int height) throws SDDMException{
            this(mod,makePB(width, height));
        }

        private static GLPbuffer makePB(int width, int height)  throws SDDMException {
            if (!GLDrawableFactory.getFactory().canCreateGLPbuffer())
                throw new SDDMException("Could not create Pbuffer - can't do selection without a supplied drawable");
            
            GLCapabilities caps = new GLCapabilities();
            caps.setRedBits(8);
            caps.setGreenBits(8);
            caps.setBlueBits(8);
            caps.setAlphaBits(8);
            GLDrawableFactory factory = GLDrawableFactory.getFactory();
            return factory.createGLPbuffer(caps, new DefaultGLCapabilitiesChooser(),width,height,null);
            
        }
        
        public void recompile() {
            this.recompile = true;
        }
        
        public void init(GLAutoDrawable drawable) {
            GL gl = drawable.getGL();
            selectDisplayList = gl.glGenLists(1);
            //TODO:test if this is enough
            mod.init(gl);
        }
        
        public void display(GLAutoDrawable drawable) {
            GL gl = drawable.getGL();
            
            if (selectDisplayList == null) 
                selectDisplayList = gl.glGenLists(1);
            
            if (recompile) {
                
                storeOld(this.mod);
                try {
                    setupModForSelect(this.mod,this.indexColorer);
                    
                    gl.glNewList(selectDisplayList,GL.GL_COMPILE);
                    mod.drawGeometry(gl);
                    gl.glEndList();
                } catch(Exception e) {
                    throw new SDDMException("Problem during selection display list generation:"+e.getMessage(),e);
                } finally {
                    
                    reapplyOld(this.mod);
                }
                recompile = false;
            }
            if (selectionNeeded) {
                
                gl.glClearColor(BGCOLOR[0],BGCOLOR[1],BGCOLOR[2],BGCOLOR[3]);
                gl.glClear(GL.GL_COLOR_BUFFER_BIT|GL.GL_DEPTH_BUFFER_BIT);
                
                StereoCamera cam = mod.getCamera();
                cam.updateCamera(gl);
                //update again to preserve stereo order for other modules 
                //TODO:major fix here to account for selecting both eyes
                if(cam.isStereo())
                    cam.updateCamera(gl);
                
                //TODO: figure out which attribs I really want to push - tis works, but I don't need them all
                gl.glPushAttrib(GL.GL_ALL_ATTRIB_BITS);
                gl.glDisable(GL.GL_LIGHTING);
                gl.glDisable(GL.GL_BLEND);
                
                if(gl.isFunctionAvailable("glUseProgram"))  
                    gl.glUseProgram(0);
                else
                    gl.glUseProgramObjectARB(0);
                 
                 gl.glCallList(selectDisplayList);
                
                /*//Don't need this because the push/pop attribs restores the program automatically - instead just the above part
                int[] shaderProg = new int[1];
                if(gl.isFunctionAvailable("glUseProgram"))  {
                    gl.glGetIntegerv(GL.GL_CURRENT_PROGRAM,shaderProg,0);
                    gl.glUseProgram(0);
                }
                else if (gl.isFunctionAvailable("glUseProgramObjectARB")) {
                    gl.glGetIntegerv(GL.GL_CURRENT_PROGRAM,shaderProg,0);
                    gl.glUseProgramObjectARB(0);
                }
                    
                gl.glCallList(selectDisplayList);
                
                
                if(gl.isFunctionAvailable("glUseProgram"))  
                    gl.glUseProgram(shaderProg[0]);
                else if (gl.isFunctionAvailable("glUseProgramObjectARB")) 
                    gl.glUseProgramObjectARB(shaderProg[0]);
                */
                
                //read out the selected pixel, get the value, and apply it to the array
                java.nio.FloatBuffer buff = com.sun.opengl.util.BufferUtil.newFloatBuffer(4);
                gl.glReadPixels(selectX,selectY,1,1,GL.GL_RGBA,GL.GL_FLOAT,buff);
                /*//Debug code
                System.err.println(buff.get()+","+buff.get()+","+buff.get()+","+buff.get());
                buff.rewind();*/
                resultf = indexColorer.getIndexedFloat(buff.get(),buff.get(),buff.get(),buff.get());
                buff.rewind();
                resulti = indexColorer.getIndexedInt(buff.get(),buff.get(),buff.get(),buff.get());
                
                gl.glPopAttrib();
                
                //selection completed - wait for another request before doing the work again
                selectionNeeded = false;
            }
        }
        
        public abstract void storeOld(SDDModule module);
        public abstract void setupModForSelect(SDDModule module, Coloring indexedColorer);
        public abstract void reapplyOld(SDDModule module);
        
        /**
         * Call this, then repaint the component in question to get selected float value stored in valueHolder
         * 
         * @param x the x-coordinate to select in window coordinates
         * @param y the y-coordinate to select in window coordinates
         * @throws java.lang.InterruptedException if the thread is interrupted while waiting for selection to be completed
         * @return the selected value at that location
         */
        public float selectFloat(int x, int y) throws InterruptedException {
            selectionNeeded = true;
            selectX = x;
            selectY = selectDrawer.getHeight()-y;
            
            if (selectDrawer instanceof GLPbuffer)
                selectDrawer.display();
            else {
                boolean swap = selectDrawer.getAutoSwapBufferMode();
                selectDrawer.setAutoSwapBufferMode(false);
                selectDrawer.display();
                selectDrawer.setAutoSwapBufferMode(swap);
            }
             if (selectionNeeded)
                throw new SDDMError("Selection not completed even after display call - something seriously wrong!");
            return resultf;
        }
        
        public float select(int x, int y) throws InterruptedException {
            return selectFloat(x,y);
        }
        
        /**
         * Call this, then repaint the component in question to get selected integer value stored in valueHolder
         * 
         * @param x the x-coordinate to select in window coordinates
         * @param y the y-coordinate to select in window coordinates
         * @throws java.lang.InterruptedException if the thread is interrupted while waiting for selection to be completed
         * @return the selected value at that location
         */
        public int selectInt(int x, int y) throws InterruptedException {
            selectionNeeded = true;
            selectX = x;
            selectY = selectDrawer.getHeight()-y;
            
            if (selectDrawer instanceof GLPbuffer)
                selectDrawer.display();
            else {
                boolean swap = selectDrawer.getAutoSwapBufferMode();
                selectDrawer.setAutoSwapBufferMode(false);
                selectDrawer.display();
                selectDrawer.setAutoSwapBufferMode(swap);
            }
            
            if (selectionNeeded)
                throw new SDDMError("selection not completed even after display call - something seriously wrong!");
            return resulti;
        }
        
        public void setSize(int width, int height) {
            if (this.selectDrawer.getWidth() != width || this.selectDrawer.getHeight() != height)
                selectDrawer.setSize(width,height);
        }

        public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
        }
        
        public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {
        }
        
    }
