/*
 * HeightFieldModuleControlPanel.java
 *
 * Created on June 21, 2006, 1:20 AM
 */

package sddm.modules;

import java.awt.event.WindowEvent;
import javax.swing.*;
import net.java.joglutils.*;
import net.java.joglutils.lighting.*;
import sddm.*;
import sddm.SDDModule;
import sddm.SDDModuleControlPanel;

/**
 *
 * @author  Erik J. Tollerud
 */
public class HeightFieldModuleControlPanel extends SDDModuleControlPanel {
    private HeightFieldModule mod;
    /** Creates new form HeightFieldModuleControlPanel */
    public HeightFieldModuleControlPanel(HeightFieldModule mod) {
        super();
        this.mod = mod;
        initComponents();
    }
    
    public HeightFieldModuleControlPanel(HeightFieldModule mod, StereoCamera cam) {
        super(cam);
        this.mod = mod;
        initComponents();
    }
    
    public void moduleSettingsToPanel(SDDModule module) {
        HeightFieldModule hfm = (HeightFieldModule)module;
        this.outlineCheckBox.setSelected(hfm.isDrawOutline());
        this.polyCheckBox.setSelected(hfm.isDrawPolygons());
        this.granField.setValue(mod.getGranularity());
        this.stippleCheckBox.setSelected(mod.isStipple());
        this.lightingCheckBox.setSelected(hfm.isLightingOn());
        this.drawNormalsCheckBox.setSelected(hfm.isDrawNormals());
        this.shininessField.setValue(hfm.getShininess());
        this.zScaleField.setValue(hfm.getZScale());
        this.phongCheckBox.setSelected(hfm.getLight().isPhongShaded());
        this.selectionCheckBox.setSelected(hfm.isSelecting());
    }
    
    public SDDModule panelToModuleSettings(SDDModule module) {
        HeightFieldModule retMod;
        if (module == null)
            retMod = new HeightFieldModule();
        else
            retMod = (HeightFieldModule)module;
        retMod.setDrawOutline(this.outlineCheckBox.isSelected());
        retMod.setDrawPolygons(this.polyCheckBox.isSelected());
        retMod.setGranularity(((Number)granField.getValue()).intValue());
        retMod.setStipple(this.stippleCheckBox.isSelected());
        retMod.setLightingOn(this.lightingCheckBox.isSelected());
        retMod.setDrawNormals(this.drawNormalsCheckBox.isSelected());
        retMod.setShininess(((Number)shininessField.getValue()).floatValue());
        retMod.setZScale(((Number)zScaleField.getValue()).floatValue());
        retMod.getLight().setPhongShaded(this.phongCheckBox.isSelected());
        if (selectionCheckBox.isSelected()) {
            JFrame[] frames = this.getControlFrame().getAttachedFrames();
            for (JFrame frame: frames) {
                if (frame instanceof GLJFrame) {
                    retMod.setSelecting(((GLJFrame)frame).getAutoDrawable());
                    break;
                }
            }
        }
        else
            retMod.setSelecting(null);

        return retMod;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        outlineColorButton = new javax.swing.JButton();
        outlineCheckBox = new javax.swing.JCheckBox();
        polyCheckBox = new javax.swing.JCheckBox();
        bgColorButton = new javax.swing.JButton();
        granField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel1 = new javax.swing.JLabel();
        stippleCheckBox = new javax.swing.JCheckBox();
        lightingButton = new javax.swing.JButton();
        lightingCheckBox = new javax.swing.JCheckBox();
        drawNormalsCheckBox = new javax.swing.JCheckBox();
        shininessField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel2 = new javax.swing.JLabel();
        zScaleField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel3 = new javax.swing.JLabel();
        colorMapButton = new javax.swing.JButton();
        phongCheckBox = new javax.swing.JCheckBox();
        editButton = new javax.swing.JButton();
        saveButton = new javax.swing.JButton();
        selectionCheckBox = new javax.swing.JCheckBox();
        selectionField = new javax.swing.JTextField();

        outlineColorButton.setText("Outline Color...");
        outlineColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outlineColorButtonActionPerformed(evt);
            }
        });

        outlineCheckBox.setSelected(mod.isDrawOutline());
        outlineCheckBox.setText("show Outline?");
        outlineCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        outlineCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        outlineCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outlineCheckBoxActionPerformed(evt);
            }
        });

        polyCheckBox.setSelected(mod.isDrawPolygons());
        polyCheckBox.setText("show Polygon mesh?");
        polyCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        polyCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        polyCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                polyCheckBoxActionPerformed(evt);
            }
        });

        bgColorButton.setText("Background Color...");
        bgColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bgColorButtonActionPerformed(evt);
            }
        });

        granField.setText("num");
        granField.setValue(mod.getGranularity());
        granField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                granFieldActionPerformed(evt);
            }
        });

        jLabel1.setText("Granularity:");

        stippleCheckBox.setSelected(mod.isStipple());
        stippleCheckBox.setText("sTipple?");
        stippleCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        stippleCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        stippleCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stippleCheckBoxActionPerformed(evt);
            }
        });

        lightingButton.setText("Configure Light...");
        lightingButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lightingButtonActionPerformed(evt);
            }
        });

        lightingCheckBox.setSelected(this.mod.isLightingOn());
        lightingCheckBox.setText("enable Lighting");
        lightingCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        lightingCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        lightingCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lightingCheckBoxActionPerformed(evt);
            }
        });

        drawNormalsCheckBox.setText("draw Normals?");
        drawNormalsCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        drawNormalsCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        drawNormalsCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                drawNormalsCheckBoxActionPerformed(evt);
            }
        });

        shininessField.setText("num");
        shininessField.setValue(mod.getShininess());
        shininessField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                shininessFieldActionPerformed(evt);
            }
        });

        jLabel2.setText("shininess:");

        zScaleField.setText("num");
        zScaleField.setValue(mod.getZScale());
        zScaleField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zScaleFieldActionPerformed(evt);
            }
        });

        jLabel3.setText("Vertical Scale:");

        colorMapButton.setText("Color Scheme...");
        colorMapButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                colorMapButtonActionPerformed(evt);
            }
        });

        phongCheckBox.setSelected(this.mod.getLight().isPhongShaded());
        phongCheckBox.setText("Phong Lighting");
        phongCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        phongCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        phongCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                phongCheckBoxActionPerformed(evt);
            }
        });

        editButton.setText("Edit Field...");
        editButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editButtonActionPerformed(evt);
            }
        });

        saveButton.setText("Save Field...");
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });

        selectionCheckBox.setSelected(mod.isSelecting());
        selectionCheckBox.setText("select?");
        selectionCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        selectionCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        selectionCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectionCheckBoxActionPerformed(evt);
            }
        });

        selectionField.setEditable(false);
        selectionField.setText("N/A");
        selectionField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectionFieldActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(outlineColorButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(outlineCheckBox)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(polyCheckBox))
                    .add(layout.createSequentialGroup()
                        .add(lightingButton)
                        .add(27, 27, 27)
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(shininessField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(lightingCheckBox))
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(bgColorButton)
                            .add(layout.createSequentialGroup()
                                .add(jLabel3)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(zScaleField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(jLabel1)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(stippleCheckBox)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(drawNormalsCheckBox))
                            .add(layout.createSequentialGroup()
                                .add(colorMapButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(phongCheckBox))))
                    .add(layout.createSequentialGroup()
                        .add(editButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(saveButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(selectionCheckBox)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(selectionField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .add(11, 11, 11))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(outlineColorButton)
                    .add(outlineCheckBox)
                    .add(polyCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(bgColorButton)
                    .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel1)
                    .add(stippleCheckBox)
                    .add(drawNormalsCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lightingButton)
                    .add(shininessField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(lightingCheckBox)
                    .add(jLabel2))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel3)
                    .add(zScaleField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(colorMapButton)
                    .add(phongCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(editButton)
                    .add(saveButton)
                    .add(selectionCheckBox)
                    .add(selectionField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(150, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    private void selectionFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectionFieldActionPerformed
        if(mod.isSelecting())
            selectionField.setText(Float.toString(mod.getSelectionResult()));
        else
            selectionField.setText("N/A");
    }//GEN-LAST:event_selectionFieldActionPerformed

    private void selectionCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectionCheckBoxActionPerformed
        javax.media.opengl.GLAutoDrawable toApply = null;
        if(selectionCheckBox.isSelected()) {
            JFrame[] frames = this.getControlFrame().getAttachedFrames();
            for (JFrame frame: frames) {
                if (frame instanceof GLJFrame) {
                    toApply = ((GLJFrame)frame).getAutoDrawable();
                    break;
                }
            }
        }
        //else leave null and it will be deactivated, and reflect the fact that its not working if it isn't'
        mod.setSelecting(toApply);
        if (toApply == null)
            selectionCheckBox.setSelected(false);
        else
            selectionFieldActionPerformed(evt);
    }//GEN-LAST:event_selectionCheckBoxActionPerformed

    private void editButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editButtonActionPerformed
        // TODO add your handling code here:
        final StereoCameraControlFrame sccf =  this.getControlFrame();
        final JFrame[] frames = sccf.getAttachedFrames();
        sddm.modules.HeightFieldEditingPanel pan = null;
        for (JFrame frame: frames) {
            if (frame instanceof sddm.modules.HeightFieldEditingPanel)
                pan = (HeightFieldEditingPanel)frame;
        }
        if(pan == null) {
            final HeightFieldEditingPanel hfep = new HeightFieldEditingPanel(mod,sccf);
            sccf.attachFrame(hfep);
            hfep.setDefaultCloseOperation(hfep.HIDE_ON_CLOSE);
            hfep.addWindowListener( new java.awt.event.WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    super.windowClosing(e);
                    hfep.setVisible(false);
                    sccf.removeFrame(hfep);
                    hfep.dispose();
                }
            });
            hfep.setVisible(true);
        } else {
            pan.setVisible(false);
            sccf.removeFrame(pan);
            pan.dispose();
        }
        
    }//GEN-LAST:event_editButtonActionPerformed

    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        JFileChooser jfc = new JFileChooser();
        jfc.setMultiSelectionEnabled(false);
        ImageIOFileFilter iioff = new ImageIOFileFilter();
        jfc.addChoosableFileFilter(iioff);
        jfc.setAcceptAllFileFilterUsed(true);
        if(jfc.showSaveDialog(this)==JFileChooser.APPROVE_OPTION) {
            try {
                if (jfc.getFileFilter() == iioff)
                    this.mod.getField().saveData(jfc.getSelectedFile(),sddm.data.HeightField.hfFileType.IMAGE);
                else {
                    if(JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(this,"Is Text File 2D (i.e. seperated by line breaks)?","Text File Type for Height Field",JOptionPane.YES_NO_OPTION))
                        this.mod.getField().saveData(jfc.getSelectedFile(),sddm.data.HeightField.hfFileType.ASCII_2D);
                    else
                        this.mod.getField().saveData(jfc.getSelectedFile(),sddm.data.HeightField.hfFileType.ASCII_1D);
                }
            } catch (Exception e) {
                JOptionPane.showMessageDialog(null,"Couldn't Save the Height Field: "+e.getMessage(),"Problem saving Height Field",JOptionPane.WARNING_MESSAGE);
            }
        };
    }//GEN-LAST:event_saveButtonActionPerformed
    
    private void phongCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_phongCheckBoxActionPerformed
        try {
            mod.getLight().setPhongShaded(this.phongCheckBox.isSelected());
            
            mod.recompileDisplayList();
            this.redisplayParent();
        } catch(LightingException e) {
            JOptionPane.showMessageDialog(null,"Problem setting Phong shader: "+e.getMessage(),"Phong Shading Failed",JOptionPane.WARNING_MESSAGE);
        }

       
    }//GEN-LAST:event_phongCheckBoxActionPerformed
    
    private void colorMapButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_colorMapButtonActionPerformed
        Coloring c = Coloring.coloringOptionPane(this);
        
        if (c!=null) {
            mod.setColorer(c);
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_colorMapButtonActionPerformed
    
    private void zScaleFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zScaleFieldActionPerformed
        mod.setZScale(((Number)zScaleField.getValue()).floatValue());
        this.redisplayParent();
    }//GEN-LAST:event_zScaleFieldActionPerformed
    
    private void shininessFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_shininessFieldActionPerformed
        try {
            mod.setShininess(((Number)shininessField.getValue()).floatValue());
            this.redisplayParent();
        } catch (SDDMException e) {
            JOptionPane.showMessageDialog(null,"Tried to set shininess greater than 128 or less than 0.  OpenGL Shininesss must be between 0 and 128, inclusive.","Couldn't set shininess",JOptionPane.ERROR_MESSAGE);
            shininessField.setValue(mod.getShininess());
        }
    }//GEN-LAST:event_shininessFieldActionPerformed
    
    private void drawNormalsCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_drawNormalsCheckBoxActionPerformed
        this.mod.setDrawNormals(drawNormalsCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_drawNormalsCheckBoxActionPerformed
    
    private void lightingCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lightingCheckBoxActionPerformed
        this.mod.setLightingOn(lightingCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_lightingCheckBoxActionPerformed
    
    private void lightingButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lightingButtonActionPerformed
        LightPanel lp = new LightPanel(mod.getLight());
        JPanelDialog jpd = new JPanelDialog(lp);
        ColorButton cb = new ColorButton("Global Ambient");
        cb.setColor(mod.getGlobalAmbient());
        jpd.add(cb);
        if(jpd.showAsModal()) {
            lp.panelToLight(mod.getLight());
            mod.setGlobalAmbient(cb.getColor());
            if (mod.isLightingOn()) {
                mod.recompileDisplayList();
                this.redisplayParent();
            }
        }
    }//GEN-LAST:event_lightingButtonActionPerformed
    
    private void stippleCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stippleCheckBoxActionPerformed
        mod.setStipple(stippleCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_stippleCheckBoxActionPerformed
    
    private void granFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_granFieldActionPerformed
        mod.setGranularity(((Number)granField.getValue()).intValue());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_granFieldActionPerformed
    
    private void bgColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bgColorButtonActionPerformed
        java.awt.Color selectedColor = new JColorChooser().showDialog(this,"Background Color?",mod.getBackgroundColor());
        if (selectedColor != null) {
            mod.setBackgroundColor(selectedColor);
            this.redisplayParent();
        }
    }//GEN-LAST:event_bgColorButtonActionPerformed
    
    private void polyCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_polyCheckBoxActionPerformed
        mod.setDrawPolygons(polyCheckBox.isSelected());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_polyCheckBoxActionPerformed
    
    private void outlineCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outlineCheckBoxActionPerformed
        mod.setDrawOutline(outlineCheckBox.isSelected());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_outlineCheckBoxActionPerformed
    
    private void outlineColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outlineColorButtonActionPerformed
        java.awt.Color selectedColor = new JColorChooser().showDialog(this,"title",mod.getOutlineColor());
        if (selectedColor != null) {
            mod.setOutlineColor(selectedColor);
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_outlineColorButtonActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bgColorButton;
    private javax.swing.JButton colorMapButton;
    private javax.swing.JCheckBox drawNormalsCheckBox;
    private javax.swing.JButton editButton;
    private javax.swing.JFormattedTextField granField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JButton lightingButton;
    private javax.swing.JCheckBox lightingCheckBox;
    private javax.swing.JCheckBox outlineCheckBox;
    private javax.swing.JButton outlineColorButton;
    private javax.swing.JCheckBox phongCheckBox;
    private javax.swing.JCheckBox polyCheckBox;
    private javax.swing.JButton saveButton;
    private javax.swing.JCheckBox selectionCheckBox;
    private javax.swing.JTextField selectionField;
    private javax.swing.JFormattedTextField shininessField;
    private javax.swing.JCheckBox stippleCheckBox;
    private javax.swing.JFormattedTextField zScaleField;
    // End of variables declaration//GEN-END:variables
    
    
    public void paint(java.awt.Graphics g) {
        java.awt.event.ActionEvent evt = new java.awt.event.ActionEvent(this,0,"");
        this.selectionFieldActionPerformed(evt);
        super.paint(g);
    }
}
