/*
 * ParametricModuleSetupPanel.java
 *
 * Created on August 2, 2006, 2:55 PM
 */

package sddm.modules;

import sddm.*;
import sddm.data.*;
import javax.swing.*;

/**
 *
 * @author  Erik J. Tollerud
 */
public class ParametricModuleSetupPanel extends SDDModuleSetupPanel {
    
    private sddm.data.ParametricSystem initialSystem;
    
    /** Creates new form ParametricModuleSetupPanel */
    public ParametricModuleSetupPanel() {
        this.initialSystem = null;
        initComponents();
    }
    
    public ParametricModuleSetupPanel(ParametricSystem initialSystem) {
        this.initialSystem = initialSystem;
        initComponents();
        int selectionIndex = 0;
        if (initialSystem != null) {
            if (initialSystem.getGen() instanceof UniformParametricGenerator) {
                UniformParametricGenerator upg = (UniformParametricGenerator)initialSystem.getGen();
                UniformParametricGenerator.Generator gen = upg.getGen();
                if (gen instanceof ParametricRosslerTypePanel.RosslerGenerator) 
                    selectionIndex = 1;
                else if(gen instanceof ParametricDiffEqTypePanel.DiffEqGenerator) 
                    selectionIndex = 2;
            }
        }
        this.typeComboBox.setSelectedIndex(selectionIndex);
        setPanelFromComboBoxSelection();
        
    }

    public SDDModule makeModule(StereoCamera camera) {
        ParametricModule pm =  new ParametricModule(makeSystem(),camera);
        pm.setGranularity(((Number)this.granField.getValue()).intValue());
        return pm;
    }
    
    
    public sddm.data.ParametricSystem makeSystem() {
        return ((TypePanel)this.typePanel.getComponent(0)).makeSystem();
    } 
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        typeComboBox = new javax.swing.JComboBox();
        jLabel1 = new javax.swing.JLabel();
        typePanel = new javax.swing.JPanel();
        this.setPanel(new ParametricLSTypePanel(this));
        jLabel2 = new javax.swing.JLabel();
        granField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getIntegerInstance());

        typeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Lorenz-Salzman", "Rossler Map", "Differential Equation" }));
        typeComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                typeComboBoxActionPerformed(evt);
            }
        });

        jLabel1.setText("Module Type");

        org.jdesktop.layout.GroupLayout typePanelLayout = new org.jdesktop.layout.GroupLayout(typePanel);
        typePanel.setLayout(typePanelLayout);
        typePanelLayout.setHorizontalGroup(
            typePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 285, Short.MAX_VALUE)
        );
        typePanelLayout.setVerticalGroup(
            typePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 277, Short.MAX_VALUE)
        );

        jLabel2.setText("Granularity");

        granField.setText("num");
        granField.setValue(1);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, typePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, layout.createSequentialGroup()
                        .add(jLabel1)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(typeComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 220, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, layout.createSequentialGroup()
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(typeComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(typePanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void typeComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_typeComboBoxActionPerformed
        setPanelFromComboBoxSelection();
    }//GEN-LAST:event_typeComboBoxActionPerformed
    protected void setPanel(JPanel panelToApply) {
        this.typePanel.removeAll();
        this.typePanel.add(panelToApply);
        panelToApply.setSize(panelToApply.getPreferredSize());
        this.typePanel.setPreferredSize(panelToApply.getPreferredSize());
        this.typePanel.setSize(panelToApply.getPreferredSize());
        this.typePanel.validate();
        
        
        java.awt.Container c = this.getTopLevelAncestor();
        if(c!=null && java.awt.Window.class.isAssignableFrom(c.getClass())) {
            java.awt.Window w = (java.awt.Window)c;
            w.pack();
            w.repaint();
        } else {
            this.typePanel.repaint();
        }
    }
    
    protected JPanel getPanel() {
        return (JPanel)this.typePanel.getComponent(0);
    }
    
    private void setPanelFromComboBoxSelection() {
        switch(this.typeComboBox.getSelectedIndex()) {
            case 0:
                this.setPanel(new ParametricLSTypePanel(this, initialSystem));
                
                break;
            case 1:
                this.setPanel(new ParametricRosslerTypePanel(this, initialSystem));
                break;
            case 2:
                this.setPanel(new ParametricDiffEqTypePanel(this, initialSystem));
                break;
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JFormattedTextField granField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JComboBox typeComboBox;
    private javax.swing.JPanel typePanel;
    // End of variables declaration//GEN-END:variables
   
    public static abstract class TypePanel extends JPanel {
        public abstract ParametricSystem makeSystem();
    }
   
}
