/*
 * Vector3DModule.java
 *
 * Created on August 28, 2006, 11:11 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm.modules;

import java.text.NumberFormat;
import javax.media.opengl.*;
import static javax.media.opengl.GL.*;
import sddm.*;
import sddm.data.*;
import javax.vecmath.*;
import javax.swing.*;
import java.util.*;

/**
 * sample: -aa 4 -mod v3m -file "C:\Documents and Settings\Erik\Desktop\module test data\VectorFields\eocto41.txt" field
 * @author Erik J. Tollerud
 */
public class Vector3DModule extends PointMap3DModule {
    public enum Model{BLOCK_ARROW,WIRE_ARROW,CONE,TORUS,CUSTOM};
    
    private PointMap<Vector4f> map;
    
    protected Model model;
    protected double[] modelDoubles;
    protected int[] modelInts;
    protected float[][] modelVerticies;
    
    private boolean zeroBaseScaling;
    protected float globalScale;
    protected float[] logFactors;
    protected float[] expFactors;
    protected float[] polyFactors;
    
    
    protected boolean drawStreamlines;
    protected List<Streamline> streamlines;
    protected ParametricModule streamlineDrawer;
    
    /** Creates a new instance of Vector3DModule */
    public Vector3DModule() {
        this(new PointMap<Vector4f>(),new StereoCamera());
    }
    
    public Vector3DModule(StereoCamera camera) {
        this(new PointMap<Vector4f>(),camera);
    }
    
    public Vector3DModule(PointMap<Vector4f> map) {
        this(map,new StereoCamera());
    }
    
    public Vector3DModule(PointMap<Vector4f> map, StereoCamera camera) {
        super(map.clone(),camera);
        this.map = (PointMap<Vector4f>)super.list;
        //TODO: figure out camera loc/targ
        //camera.setCameraLocation(map.getXMax(),map.getYMax(),map.getZMax());
        //camera.setTargetPoint(map.getXMin(),map.getYMin(),map.getZMin());
        
        this.useBlockArrowModel();
        zeroBaseScaling = true;
        this.setGlobalScaleToFillBox(0.5f);
        this.setBackgroundColor(new java.awt.Color(0.65f,0.65f,0.65f,1.0f));
        
        this.drawStreamlines = false;
        this.streamlines = new Vector<Streamline>();
        this.streamlineDrawer = new ParametricModule(new ParametricSystem());
        this.streamlineDrawer.setCamera(cam);
    }
    
    protected float[] determineColor(Coloring colorer, int index) {
        return colorer.getColorByValue(this.map.get(index).length());
    }
    
    //private static final Vector3f xHat = new Vector3f(1.0f,0,0);
    private static final Vector3f zHat = new Vector3f(0,0,1.0f);
    private static Vector3f manipVec = new Vector3f();
    private static Vector3f rotVec = new Vector3f();
    
    protected void manipulatePostTranslate(GL gl, int index) {
        //TODO:see how the 3d vs 4d differences affect things
        Vector4f vec4 = this.map.get(index);
        manipVec.set(vec4.x,vec4.y,vec4.z);
        
        float val = manipVec.length();
        float offset = 0.0f;
        
        try {
            if(!this.zeroBaseScaling)
                offset = (this.map.getMinValue().length() - 1);// the - 1 accounts for the 4D internal vector storage
        } catch (NoSuchMethodException e) {
            throw new SDDMError("Could not get length from Vector3DModule",e);
        }
        
        float scaleVal = val-offset;
        
        if (this.logFactors != null) 
            scaleVal = (float)Math.log(logFactors[0]*scaleVal+logFactors[1])+logFactors[2];
        else if (this.expFactors != null)
            scaleVal = (float)Math.exp(expFactors[0]*scaleVal+expFactors[1])+expFactors[2];
        else if (this.polyFactors != null) {
            float x = scaleVal;
            float currPoly = 1;
            for(float coeff : this.polyFactors) {
                scaleVal+=(coeff*currPoly);
                currPoly*=x;
            }
        }
        
        scaleVal *= this.globalScale;
        gl.glScalef(scaleVal,scaleVal,scaleVal);
        
        
        rotVec.cross(zHat,manipVec);
        float rotAngle = (float)(180.0*Math.acos((zHat.dot(manipVec))/(manipVec.length()*zHat.length()))/Math.PI);
        gl.glRotatef(rotAngle,rotVec.x,rotVec.y,rotVec.z);
    }
    
    public String getSelectionInfoString(NumberFormat indexnf, NumberFormat valnf, String delim) {
        try {
            int indx = this.getSelectionIndex();
            float[] selPoint = this.map.getPoint(indx);
            Vector4f selVec = this.map.get(indx);
            
            return "x:"+indexnf.format(selPoint[0])+" y:"+indexnf.format(selPoint[1])+" z:"+indexnf.format(selPoint[2])+
                    "Vector:x:"+valnf.format(selVec.x)+" y:"+valnf.format(selVec.y)+" z:"+valnf.format(selVec.z)+" w:"+valnf.format(selVec.w);
        } catch (Exception e) {
            return "Invalid Selection";
        }
    }
    
    public void drawGeometry(GL gl) {
        super.drawGeometry(gl);
        if (this.drawStreamlines) {
            gl.glDisable(GL.GL_LIGHTING);
            if (gl.isExtensionAvailable("glUseProgram"))
                gl.glUseProgram(0);
            else if (gl.isExtensionAvailable("glUseProgramObjectARB"))
                gl.glUseProgramObjectARB(0);
            
            for (Streamline sl : this.streamlines) {
                streamlineDrawer.setSystem(sl);
                streamlineDrawer.preDraw(gl);
                streamlineDrawer.drawGeometry(gl);
            }
        }
    }
    
    protected void drawModel(GL gl) {
        switch(this.model) {
            case BLOCK_ARROW:
                //begin: Draw Arrow pointed along x-axis
                //draw cylinder from [-1,0,0] to [0.5,0,0] w/ radius 0.25 (can ignore top chunk)
                //draw cone with radius .5 with tip at [1,0,0] and base centered at [0.5,0,0]
                /*int numOfSides = 8;
                float[] cylY = new float[numOfSides];
                float[] cylZ = new float[numOfSides];
                //generate cylinder edges
                float trigFactor = (float)(2.0*Math.PI/numOfSides);
                for (int i = 0; i < numOfSides; i++) {
                    cylY[i] = (float)(0.25 * Math.sin(i*trigFactor));
                    cylZ[i] = (float)(0.25 * Math.cos(i*trigFactor));
                }
                gl.glBegin(GL_TRIANGLE_FAN);
                //These normals should all be straight down for the butt-end of the arrow
                gl.glNormal3f(-1,0,0);
                gl.glVertex3f(-1.0f,0.0f,0.0f);
                for (int i = 0; i < numOfSides; ++i) {
                    gl.glVertex3f(-1.0f,cylY[i],cylZ[i]);
                }
                gl.glVertex3f(-1.0f,cylY[0],cylZ[0]);
                gl.glEnd();
                gl.glBegin(GL_QUAD_STRIP);
                for (int i = 0; i < numOfSides; ++i) {
                    gl.glNormal3f(0,4*cylY[i],4*cylZ[i]);
                    gl.glVertex3f(-1.0f,cylY[i],cylZ[i]);
                    gl.glVertex3f(0.5f,cylY[i],cylZ[i]);
                }
                gl.glNormal3f(0,4*cylY[0],4*cylZ[0]);
                gl.glVertex3f(-1.0f,cylY[0],cylZ[0]);
                gl.glVertex3f(0.5f,cylY[0],cylZ[0]);
                gl.glEnd();
                //Arrow parts done twice for cleanliness of backwards-looking
                //Arrow head
                gl.glBegin(GL_TRIANGLE_FAN);
                gl.glNormal3f(1,0,0);
                gl.glVertex3f(1.0f,0.0f,0.0f);
                gl.glNormal3f(-0.25f,cylY[0],cylZ[0]);
                gl.glVertex3f(0.5f,2*cylY[0],2*cylZ[0]);
                for (int i = numOfSides - 1; i >= 0; --i) {
                    gl.glNormal3f(-0.25f,cylY[i],cylZ[i]);
                    gl.glVertex3f(0.5f,2*cylY[i],2*cylZ[i]);
                }
                gl.glEnd();
                //Arrow head base
                gl.glBegin(GL_TRIANGLE_FAN);
                //Again, these normals all face toward the back of the arrow
                gl.glNormal3f(-1,0,0);
                gl.glVertex3f(0.5f,0.0f,0.0f);
                for (int i = 0; i < numOfSides; ++i) {
                    gl.glVertex3f(0.5f,2*cylY[i],2*cylZ[i]);
                }
                gl.glVertex3f(0.5f,2*cylY[0],2*cylZ[0]);
                gl.glEnd();
                //end of arrow
                 */
                //begin: Draw Arrow pointed along z-axis
                //draw cylinder from [0,0,-1] to [0,0,0.5] w/ radius 0.25 (can ignore top chunk)
                //draw cone with radius .5 with tip at [0,0,1] and base centered at [0,0,0.5]
                int numOfSides = 8;
                float[] cylX = new float[numOfSides];
                float[] cylY = new float[numOfSides];
                //generate cylinder edges
                float trigFactor = (float)(2.0*Math.PI/numOfSides);
                for (int i = 0; i < numOfSides; i++) {
                    cylX[i] = (float)(0.25 * Math.sin(i*trigFactor));
                    cylY[i] = (float)(0.25 * Math.cos(i*trigFactor));
                }
                gl.glBegin(GL_TRIANGLE_FAN);
                //These normals should all be straight down for the butt-end of the arrow
                gl.glNormal3f(0,0,-1);
                gl.glVertex3f(0.0f,0.0f,-1.0f);
                for (int i = 0; i < numOfSides; ++i) {
                    gl.glVertex3f(cylX[i],cylY[i],-1.0f);
                }
                gl.glVertex3f(cylX[0],cylY[0],-1.0f);
                gl.glEnd();
                gl.glBegin(GL_QUAD_STRIP);
                for (int i = 0; i < numOfSides; ++i) {
                    gl.glNormal3f(4*cylX[i],4*cylY[i],0);
                    gl.glVertex3f(cylX[i],cylY[i],-1.0f);
                    gl.glVertex3f(cylX[i],cylY[i],0.5f);
                }
                gl.glNormal3f(4*cylX[0],4*cylY[0],0);
                gl.glVertex3f(-cylX[0],cylY[0],-1.0f);
                gl.glVertex3f(cylX[0],cylY[0],0.5f);
                gl.glEnd();
                //Arrow parts done twice for cleanliness of backwards-looking
                //Arrow head
                gl.glBegin(GL_TRIANGLE_FAN);
                gl.glNormal3f(0,0,1);
                gl.glVertex3f(0.0f,0.0f,1.0f);
                gl.glNormal3f(cylX[0],cylY[0],-0.25f);
                gl.glVertex3f(2*cylX[0],2*cylY[0],0.5f);
                for (int i = numOfSides - 1; i >= 0; --i) {
                    gl.glNormal3f(cylX[i],cylY[i],-0.25f);
                    gl.glVertex3f(2*cylX[i],2*cylY[i],0.5f);
                }
                gl.glEnd();
                //Arrow head base
                gl.glBegin(GL_TRIANGLE_FAN);
                //Again, these normals all face toward the back of the arrow
                gl.glNormal3f(0,0,-1);
                gl.glVertex3f(0.0f,0.0f,0.5f);
                for (int i = 0; i < numOfSides; ++i) {
                    gl.glVertex3f(2*cylX[i],2*cylY[i],0.5f);
                }
                gl.glVertex3f(2*cylX[0],2*cylY[0],0.5f);
                gl.glEnd();
                //end of arrow
                break;
            case WIRE_ARROW:
                //draw lines from [0,0,-1] to [0,0,1] and [0,-0.5,0] to [0,0,1] and [0,0.5,0] to [0,0,1]
                gl.glBegin(GL_LINES);
                gl.glVertex3f(0,0,-1.0f);
                gl.glVertex3f(0,0,1.0f);
                gl.glVertex3f(0,-0.5f,0);
                gl.glVertex3f(0,0,1.0f);
                gl.glVertex3f(0,0.5f,0);
                gl.glVertex3f(0,0,1.0f);
                gl.glEnd();
                break;
            case CONE:
                glut.glutSolidCone(this.modelDoubles[0],this.modelDoubles[1],this.modelInts[0],this.modelInts[1]);
                break;
            case TORUS:
                glut.glutSolidTorus(this.modelDoubles[0],this.modelDoubles[1],this.modelInts[0],this.modelInts[1]);
                break;
            case CUSTOM:
                gl.glBegin(GL_POLYGON);
                for(float[] f : this.modelVerticies)
                    gl.glVertex4fv(f,0);
                gl.glEnd();
                break;
        }
    }
    
    public void applyColorBounds(Coloring colorer) {
        try {
            colorer.setTop(this.map.getMaxValue().length());
            colorer.setBottom(this.map.getMinValue().length());
        } catch (NoSuchMethodException e) {
            throw new SDDMError("Serious problem; can't order Vector3DModule's map elements",e);
        }
    }
    
    protected Float getSelectionValue() {
        try {
            return this.map.get(this.getSelectionIndex()).length();
        } catch (Exception e) {
            return null;
        }
    }
    
    public SDDModuleControlPanel newModuleControlPanel() {
        return new Vector3DModuleControlPanel(this);
    }
    
    public static SDDModuleSetupPanel newStartupPanel() {
        return new Vector3DModuleSetupPanel();
    }
    
    public void useBlockArrowModel() {
        this.model = Model.BLOCK_ARROW;
        modelDoubles = null;
        modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useWireArrowModel() {
        this.model = Model.WIRE_ARROW;
        modelDoubles = null;
        modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public void useConeModel(double baseToHeightRatio, int slices, int stacks) {
        this.model = Model.CONE;
        this.modelDoubles = new double[2];
        this.modelInts = new int[2];
        if (baseToHeightRatio > 1.0) {
            modelDoubles[0] = 1.0;//base
            modelDoubles[1] = 1.0/baseToHeightRatio;//height
        } else {
            modelDoubles[0] = baseToHeightRatio;//base
            modelDoubles[1] = 1.0;//height
        }
        modelInts[0] = slices;
        modelInts[1] = stacks;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    /**
     * Uses a torus to represent the points in question
     * @param innerToOuterRadiusRatio should be less than 1
     * @param nsides the number of sides to use in generating the torus' polygons
     * @param rings the number of annular rings to use in generating the torus' polygons
     */
    public void useTorusModel(double innerToOuterRadiusRatio, int nsides, int rings) {
        this.model = Model.TORUS;
        this.modelDoubles = new double[2];
        this.modelInts = new int[2];
        modelDoubles[0] = 1.0 - innerToOuterRadiusRatio;//innerRadius
        modelDoubles[1] = 1.0;//OuterRadius
        modelInts[0] = nsides;
        modelInts[1] = rings;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    /**
     * The direction of arrow pointing is down the positive z-axis
     * @param verticies a list of 3 or 4-element points
     * @throws sddm.SDDMException if the points are not 3 or 4 elements long
     */
    public void useCustomModel(float[][] verticies) throws SDDMException{
        //TODO:test
        this.model = Model.CUSTOM;
        this.modelDoubles = null;
        this.modelInts = null;
        this.modelVerticies = new float[verticies.length][4];
        for (int i = 0; i < verticies.length; ++i) {
            float[] f = verticies[i];
            if (f.length == 3) {
                this.modelVerticies[i][0] = verticies[i][0];
                this.modelVerticies[i][1] = verticies[i][1];
                this.modelVerticies[i][2] = verticies[i][2];
                this.modelVerticies[i][3] = 1.0f;
            } else if (f.length == 4) {
                this.modelVerticies[i][0] = verticies[i][0];
                this.modelVerticies[i][1] = verticies[i][1];
                this.modelVerticies[i][2] = verticies[i][2];
                this.modelVerticies[i][3] = verticies[i][3];
            } else
                throw new SDDMException("Supplied vertex is not 3 or 4 elements long.");
        }
        
        this.modelDoubles = null;
        this.modelInts = null;
        this.regenerateModels = true;
        this.recompileDisplayList();
    }
    
    public float[][] getCustomModel() {
        //TODO: test if this is deep or shallow
        return this.modelVerticies.clone();
    }
    
    public Model getModelType() {
        return this.model;
    }
    
    void setMap(PointMap<Vector4f> newMap) {
        this.map = newMap;
        super.list = newMap;
    }
    
    PointMap<Vector4f> getMap() {
        return this.map;
    }
    
    public static String moduleTitle() {
        return "Vector 3D Drawing Module";
    }
    
    public static SDDModule makeModuleFromArgs(String modName, java.util.HashMap<String,Integer> switchMap, String[] args, StereoCamera camera) throws  ParsingException{
        //takes modName v3m or vector3dmodule
        //-(f|file) file [four] [field]
        //-[g|gran] #
        //-[colorol|olcolor]
        //
        if (modName.equals("v3m") || modName.equals("vector3dmodule") ) {
            Integer fileIndx = switchMap.remove("-f");
            if( fileIndx == null )
                fileIndx = switchMap.remove("-file");
            if ( fileIndx == null)
                throw new ParsingException("No data file provided for Scalar3DModule");
            
            java.io.File f = new java.io.File(args[fileIndx+1]);
            boolean threeElement = true;
            boolean field = false;
            if((args.length > fileIndx+2) && args[fileIndx+2].equals("four"))
                threeElement = false;
            if((args.length > fileIndx+2) && args[fileIndx+2].equals("field"))
                field = true;
            if((args.length > fileIndx+3) && args[fileIndx+3].equals("four"))
                threeElement = false;
            if((args.length > fileIndx+3) && args[fileIndx+3].equals("field"))
                field = true;
            
            Vector3DModule v3m;
            try {
                v3m = new Vector3DModule(PointMap.loadPointsAndFloatVectorsFromFile(f,threeElement,field),camera);
            } catch (FileFormatException e) {
                JOptionPane.showMessageDialog(null,"File is not corect format:"+e.getMessage()+" Using empty PointMap","File Incorrect Format",JOptionPane.ERROR_MESSAGE);
                v3m = new Vector3DModule(camera);
            } catch (java.io.FileNotFoundException e) {
                JOptionPane.showMessageDialog(null,"No File With Name "+f+", using empty PointMap","File Not Found",JOptionPane.ERROR_MESSAGE);
                v3m = new Vector3DModule(camera);
            }
            
            Integer granIndex = switchMap.remove("-g");
            if (granIndex == null)
                granIndex = switchMap.remove("-gran");
            if (granIndex != null) {
                String granStr = null;
                try {
                    granStr = args[granIndex+1];
                    v3m.setGranularity(Integer.getInteger(granStr));
                } catch (Exception e) {
                    throw new ParsingException("Couldn't understand granularity command "+granStr+":"+e.getMessage(),e);
                }
                //TODO:implement more command line options
            }
            
            Integer olcIndex = switchMap.remove("-olcolor");
            if (olcIndex == null)
                olcIndex = switchMap.remove("-colorol");
            if (olcIndex != null) {
                v3m.polygonColorer.setColorSchemeToUniform(new java.awt.Color(1.0f,1.0f,1.0f,1.0f));
                v3m.outlineColorer.setColorSchemeToPsychadelic();
                v3m.setDrawOutline(true);
                v3m.setBackgroundColor(new java.awt.Color(0.25f,0.25f,0.25f,1.0f));
            }
            
            return v3m;
            
            
        }
        return null;
    }
    
    public void setGlobalScale(float globalScale) {
        this.globalScale = globalScale;
    }
    
    public void setGlobalScaleToFillBox(float boxSize) {
        try {
            float max = this.map.getMaxValue().length();
            float min = this.map.getMinValue().length();
            
            max = Math.max(Math.abs(max),Math.abs(min));
            this.setGlobalScale(boxSize/max);
            this.setLinearScaling();
        } catch (NoSuchMethodException e) {
            throw new SDDMError("Could not get value from Scalar3DModule",e);
        }
    }
    
    public float getGlobalScale() {
        return this.globalScale;
    }
    
    public void setZeroBaseScaling(boolean zeroBase) {
        this.zeroBaseScaling = zeroBase;
        this.recompileDisplayList();
    }
    
    public boolean isZeroBaseScaling() {
        return this.zeroBaseScaling;
    }
    
    public void setLinearScaling() {
        this.expFactors = null;
        this.logFactors = null;
        this.polyFactors = null;
    }
    
    public boolean isLinearScaling() {
        return (this.expFactors == null && this.logFactors == null && this.polyFactors == null);
    }
    
    public void setLogarithmicScaling() {
        this.setLogarithmicScaling(1,1,0);
    }
    
    public void setLogarithmicScaling(float gain, float inputOffset,float outputOffset) {
        this.logFactors = new float[3];
        this.logFactors[0] = gain;
        this.logFactors[1] = inputOffset;
        this.logFactors[2] = outputOffset;
        this.expFactors = null;
        this.polyFactors = null;
    }
    
    public boolean isLogarithmicScaling() {
        return this.logFactors != null;
    }
    
    public void setExponentialScaling() {
        this.setExponentialScaling(1,0,-1);
    }
    
    public void setExponentialScaling(float gain, float inputOffset,float outputOffset) {
        this.expFactors = new float[3];
        this.expFactors[0] = gain;
        this.expFactors[1] = inputOffset;
        this.expFactors[2] = outputOffset;
        this.logFactors = null;
        this.polyFactors = null;
    }
    
    public boolean isExponentialScaling() {
        return this.expFactors != null;
    }
    
    /**
     * Scales from a polynomial function
     * @param coefficients the coefficient of a term of the polynomial, starting at 0th order
     */
    public void setPolynomialScaling(float[] coefficients) {
        this.polyFactors = coefficients;
        this.logFactors = null;
        this.expFactors = null;
    }
    
    public void setPolynomialScaling(float quadratic) {
        float[] coeffs = {0,0,quadratic};
        this.polyFactors = coeffs;
        this.logFactors = null;
        this.expFactors = null;
    }
    
    public void setPolynomialScaling(float quadratic, float cubic) {
        float[] coeffs = {0,0,quadratic,cubic};
        this.polyFactors = coeffs;
        this.logFactors = null;
        this.expFactors = null;
    }
    
    public boolean isPolynomialScaling() {
        return this.polyFactors != null;
    }
    
    /**
     * Generates a streamline from the vectors in this point and the
     * @param initialPoint the location to start the streamline from as an {x,y,z} or {x,y,z,w} array.
     * @throws sddm.SDDMException if the supplied point is not 3-element or 4-element
     */
    public void addStreamline(float[] initialPoint) throws SDDMException {
        this.streamlines.add(new Streamline(initialPoint));
        this.recompileDisplayList();
    }
    
    public Streamline[] getStreamlines() {
        Streamline[] sls = new Streamline[this.streamlines.size()];
        return this.streamlines.toArray(sls);
    }
    
    public void clearStreamlines() {
        this.streamlines.clear();
    }
    
    public void removeStreamline(int index) {
        this.streamlines.remove(index);
    }
    
    public void setStreamlineColorer(Coloring slColorer) {
        this.streamlineDrawer.setColorer(slColorer);
    }
    
    public Coloring getStreamlineColorer() {
        return this.streamlineDrawer.getColorer();
    }
    
    public void setDrawStreamlines(boolean drawStreamlines) {
        this.drawStreamlines = drawStreamlines;
    }
    
    public boolean isDrawStreamlines() {
        return this.drawStreamlines;
    }
    
    public void setGranularity(int granularity) {
        this.streamlineDrawer.setGranularity(granularity);
    }
    
    public void setStreamlineGranularity(int gran) {
        this.streamlineDrawer.setGranularity(gran);
    }
    
    public int getStreamlineGranularity() {
        return this.streamlineDrawer.getGranularity();
    }
    
    public void setStreamlineWidth(float lineWidth) {
        streamlineDrawer.setDrawingSize(lineWidth);
    }
    
    public float getStreamlineWidth() {
        return streamlineDrawer.getDrawingSize();
    }
    
    public void recomputeStreamlines() {
        for (int i = 0; i < this.streamlines.size();++i) {
            Streamline sl = streamlines.get(i);
            this.streamlines.set(i,new Streamline(sl.getICs(),sl.getIterLimit(),sl.isStopOnStuck()));
        }
    }
    public class Streamline extends ParametricSystem {
        private int iterLimit;
        private boolean stopOnStuck;
        
        /**
         * Generates a streamline with the given initial Point out to the edge of this vector point set, or it gets stuck between two vectors.
         * @param initialPoint the location to start the streamline from as an {x,y,z} or {x,y,z,w} array.
         * @throws sddm.SDDMException if the supplied point is not 3-element or 4-element
         */
        Streamline(float[] initialPoint) throws SDDMException {
            this(initialPoint,(int)(10*(map.getXMax() - map.getXMin() + map.getYMax() - map.getYMin() + map.getZMax() - map.getZMin())),true);
            //uses a number of iterations intended to fill out the sum of the number of "cells" in the system (i.e. uniformly-spaced cubes)
        }
        
        /**
         * Generates a streamline with the given initial Point out to the edge of this vector point set.
         * 
         * @param iterLimit the maximum number of iterations to go before stopping
         * @param initialPoint the location to start the streamline from as an {x,y,z} or {x,y,z,w} array.
         * @param stopOnStuck if true, a streamline runs until it bounces back and forth 5 times.
         * @throws sddm.SDDMException 
         */
        Streamline(float[] initialPoint, int iterLimit, boolean stopOnStuck) throws SDDMException {
            super(new StreamlineGenerator(initialPoint.clone()));
            this.iterLimit = iterLimit;
            this.stopOnStuck = stopOnStuck;
            while (isInside() && (!isStuck())&& this.getCurrentIteration() < iterLimit)
                this.iterate();
        }
        
        /**
         * Generates a streamline with the given initial point through a particular number of iterations.
         *
         * @param numberOfSteps the number of iterations to perform in generating this streamline
         * @param iterLimit the location to start the streamline from as an {x,y,z} or {x,y,z,w} array.
         * @throws sddm.SDDMException if the supplied point is not 3-element or 4-element
         */
        Streamline(float[] initialPoint, int iterLimit) throws SDDMException {
            this(initialPoint,iterLimit,false);
            
        }
        
        public boolean isStopOnStuck() {
            return this.stopOnStuck;
        }
        
        public int getIterLimit() {
            return this.iterLimit;
        }
        
        //tells if the current point is within the bounding box of this set of points
        public boolean isInside() {
            float[] last = this.getPointList().getLastPoint();
            boolean inX =  map.getXMax() > last[0] && map.getXMin() < last[0];
            boolean inY =  map.getXMax() > last[1] && map.getXMin() < last[1];
            boolean inZ =  map.getXMax() > last[2] && map.getXMin() < last[2];
            boolean inW;
            if (((StreamlineGenerator)this.getGen()).isThreeD())
                inW = true;
            else
                inW = map.getXMax() > last[3] && map.getXMin() < last[3];
            
            return inX && inY && inZ && inW;
        }
        
        //tells if the generator is stuck
        public boolean isStuck() {
            StreamlineGenerator sg = (StreamlineGenerator)this.getGen();
            return sg.numberOfIndexOscillations > sg.stuckNumber;
        }
        
        /**
         * Retrieves the single-point initial conditions of this generator
         * @return a clone of the ICs used - may be 3-element or 4-element
         */
        public float[] getICs() {
            return ((StreamlineGenerator)this.getGen()).init.clone();
        }
        
        public void setGen(ParametricGenerator gen) {
            throw new SDDMException("Tried to set Parametric Generator on Streamline!");
        }
    }
    
    private class StreamlineGenerator implements ParametricGenerator {
        final int stuckNumber;
        float[] init;
        boolean stuck;
        
        //Does not perform copy/clone
        private StreamlineGenerator(float[] initialPoint, int stuckNum) {
            stuckNumber = stuckNum;
            if (initialPoint.length < 3 && initialPoint.length > 4)
                throw new SDDMException("Supplied Initial Point for Streamline is not 3D or 4D");
            init = initialPoint;
            stuck = false;
        }
        
        private StreamlineGenerator(float[] initialPoint) {
            this(initialPoint,10);
        }
        
        public boolean isThreeD() {
            return (init.length == 3);
        }
        
        public float[] getICs() {
            return init;
        }
        
        private int lastIndex = 0;
        private int lastDifferentIndex = -1;
        private int numberOfIndexOscillations = 0;
        public float[] nextIteration(Vector<Float> time, PointList wholeList) {
            float[] result= new float[4];
            
            //TODO:optimize - biggest hogs by far are ArrayList.get(index) and PointList.getPoint(index)
            
            if (isThreeD()) {
                
                //find closest point
                int closestIndex = 0;
                float closestDistSquared = Float.MAX_VALUE;
                float[] last = wholeList.getLastPoint();
                
                for(int i = 0; i < map.size(); ++i) {
                    float[] pt = map.getPoint(i);
                    float xSep = pt[0] - last[0];
                    float ySep = pt[1] - last[1];
                    float zSep = pt[2] - last[2];
                    float currDistSquared = xSep*xSep+ySep*ySep+zSep*zSep;
                    if ( currDistSquared < closestDistSquared) {
                        closestIndex = i;
                        closestDistSquared = currDistSquared;
                    }
                    
                }
                
                //get vector at closest point
                Vector4f local4 = map.get(closestIndex);
                Vector3f localDirection = new Vector3f(local4.x,local4.y,local4.z);
                localDirection.normalize();
                float minsz = getMinFeatureSizeHint();
                localDirection.scale(minsz);
                
                //add normalized and scaled result
                result[0] = last[0]+localDirection.x;
                result[1] = last[1]+localDirection.y;
                result[2] = last[2]+localDirection.z;
                
                result[3] = 1.0f;
                
                if (closestIndex != lastIndex) {
                    if (lastDifferentIndex == closestIndex)
                        numberOfIndexOscillations++;
                    else
                        numberOfIndexOscillations = 0;
                    lastDifferentIndex = lastIndex;
                }
                lastIndex = closestIndex;
            } else {
                //find closest point
                int closestIndex = 0;
                float closestDistSquared = Float.MAX_VALUE;
                float[] last = wholeList.getLastPoint();
                
                for(int i = 0; i < map.size(); ++i) {
                    float[] pt = map.getPoint(i);
                    float xSep = pt[0] - last[0];
                    float ySep = pt[1] - last[1];
                    float zSep = pt[2] - last[2];
                    float wSep = pt[3] - last[3];
                    float currDistSquared = xSep*xSep+ySep*ySep+zSep*zSep+wSep*wSep;
                    if ( currDistSquared < closestDistSquared) {
                        closestIndex = i;
                        closestDistSquared = currDistSquared;
                    }
                    
                }
                //get vector at closest point
                Vector4f localDirection = map.get(closestIndex);
                localDirection.normalize();
                localDirection.scale(getMinFeatureSizeHint());
                
                result[0] = last[0]+localDirection.x;
                result[1] = last[1]+localDirection.y;
                result[2] = last[2]+localDirection.z;
                result[3] = last[3]+localDirection.w;
                
                if (closestIndex != lastIndex) {
                    if (lastDifferentIndex == closestIndex)
                        numberOfIndexOscillations++;
                    else
                        numberOfIndexOscillations = 0;
                    lastDifferentIndex = lastIndex;
                }
                lastIndex = closestIndex;
            }
            
            return result;
        }
        
        public float[] initialZs() {
            float[] res = new float[1];
            res[0] = init[2];
            return res;
        }
        
        public float[] initialYs() {
            float[] res = new float[1];
            res[0] = init[1];
            return res;
        }
        
        public float[] initialXs() {
            float[] res = new float[1];
            res[0] = init[0];
            return res;
        }
        
        public float[] initialWs() {
            if (this.isThreeD()) {
                float[] ret = {1.0f};
                return ret;
            }
            float[] res = new float[1];
            res[0] = init[4];
            return res;
        }
        
        public float[] initialTs() {
            float[] ret = {0.0f};
            return ret;
        }
    }
    
}
